"""The `engine` module provides :class:`CommandEngine` for executing
:class:`hooke.command.Command`\s.
"""

from .ui import CloseEngine, CommandMessage

class CommandEngine (object):
    def run(self, hooke, ui_to_command_queue, command_to_ui_queue):
        """Get a :class:`hooke.ui.QueueMessage` from the incoming
        `ui_to_command_queue` and act accordingly.

        If the message is a :class:`hooke.ui.CommandMessage` instance,
        the command run may read subsequent input from
        `ui_to_command_queue` (if it is an interactive command).  The
        command may also put assorted data into `command_to_ui_queue`.

        When the command completes, it will put a
        :class:`hooke.command.CommandExit` instance into
        `command_to_ui_queue`, at which point the `CommandEngine` will
        be ready to receive the next :class:`hooke.ui.QueueMessage`.
        """
        while True:
            msg = ui_to_command_queue.get()
            if isinstance(msg, CloseEngine):
                break
            assert isinstance(msg, CommandMessage), type(msg)
            msg.command.run(hooke, ui_to_command_queue, command_to_ui_queue,
                            **msg.arguments)
